@@ -1,4 +1,3 @@
-import base64
 import os
 import re
 import json
@@ -9,7 +8,6 @@ from cloudpathlib import S3Client
 from botocore.exceptions import ClientError
 from cmapPy.pandasGEXpress.parse_gct import parse
 from os import environ
-from jose import jwt
 from polly.errors import (
     BadRequestError,
     error_handler,
@@ -23,13 +21,14 @@ from polly.errors import (
     DatatypeNotFoundException,
     RepositoryNotFoundException,
 )
-from polly.constants import COHORT_CONSTANTS_URL, REPORT_FIELDS_URL, ELUCIDATA_LOGO_URL
-from bs4 import BeautifulSoup
+from polly.constants import COHORT_CONSTANTS_URL
 import contextlib
 import joblib
 import urllib
 import pandas as pd
 import polly.http_response_codes as http_codes
+from polly.tracking import Track
+import polly.constants as const
 
 
 def get_platform_value_from_env(
@@ -59,6 +58,32 @@ def make_path(prefix: any, postfix: any) -> str:
     return os.path.normpath(f"{prefix}/{postfix}")
 
 
+def debug_print(self, val: str):
+    """Helper function to show prints in test and dev environment
+
+    Args:
+        self (polly_session_object): polly_session
+        val (str): value to be printed
+    """
+    if self.session.env != const.PROD_ENV_NAME:
+        print(val)
+
+
+@Track.track_decorator
+def debug_logger(self, properties: dict):
+    """Track an event but calling the debug logger with properties
+    that needs to be tracked
+    For Example :-
+    If I need to track the Crash of an API and properties related to it
+    like which API is crashing, for which page size it crashed and other
+    relevant details
+
+    Args:
+        properties (dict): Properties of the event which is tracked
+    """
+    return properties
+
+
 def get_sts_creds(sts_dict: dict) -> dict:
     """
     Function to check and return temporary sts creds
@@ -133,7 +158,10 @@ def upload_to_S3(cloud_path: str, local_path: str, credentials: dict) -> None:
         aws_access_key_id=access_key_id,
         aws_secret_access_key=secret_access_key,
         aws_session_token=session_token,
+        extra_args={"ContentType": "text/html"},
     )
+    # use these extras for all CloudPaths
+    client.set_as_default_client()
     source_path = client.CloudPath(cloud_path)
     if not source_path.exists():
         source_path.mkdir()
@@ -143,7 +171,9 @@ def upload_to_S3(cloud_path: str, local_path: str, credentials: dict) -> None:
         raise OperationFailedException(e)
 
 
-def download_from_S3(cloud_path: str, workspace_path: str, credentials: dict) -> None:
+def download_from_S3(
+    cloud_path: str, workspace_path: str, credentials: dict, destination_path: str
+) -> None:
     """
     Function to download file/folder from workspaces
     """
@@ -161,10 +191,7 @@ def download_from_S3(cloud_path: str, workspace_path: str, credentials: dict) ->
     isFile = source_path.is_file()
     if isFile:
         try:
-            dest_path = os.getcwd()
-            source_path.copy(dest_path, force_overwrite_to_cloud=True)
-            logging.basicConfig(level=logging.INFO)
-            logging.info(f"Download successful to path={dest_path}")
+            source_path.copy(destination_path, force_overwrite_to_cloud=True)
         except ClientError as e:
             raise OperationFailedException(e)
     else:
@@ -174,10 +201,8 @@ def download_from_S3(cloud_path: str, workspace_path: str, credentials: dict) ->
         if not source_path.is_dir():
             raise InvalidPathException
         try:
-            dest_path = f"{make_path(os.getcwd(),workspace_path)}"
-            source_path.copytree(dest_path, force_overwrite_to_cloud=True)
-            logging.basicConfig(level=logging.INFO)
-            logging.info(f"Download successful to path={dest_path}")
+            destination_path = f"{make_path(destination_path,workspace_path)}"
+            source_path.copytree(destination_path, force_overwrite_to_cloud=True)
         except ClientError as e:
             raise OperationFailedException(e)
 
@@ -303,6 +328,7 @@ def get_data_type(self, url: str, payload: dict) -> str:
     return data_type
 
 
+# used in move data
 def get_metadata(self, url: str, payload: dict) -> str:
     """
     Function to return the data-type of the required dataset
@@ -324,6 +350,7 @@ def get_metadata(self, url: str, payload: dict) -> str:
     return dataset
 
 
+# used in move dataset
 def elastic_query(index_name: str, dataset_id: str) -> dict:
     query = {
         "query": {
@@ -347,15 +374,6 @@ def get_cohort_constants() -> json:
     return json.loads(response.text)
 
 
-def get_cohort_fields() -> json:
-    """
-    Returns file format info from public assests url
-    """
-    response = requests.get(REPORT_FIELDS_URL)
-    error_handler(response)
-    return json.loads(response.text)
-
-
 def validate_datatype(datatype: str):
     """
     Function to validate datatype of a dataset
@@ -388,138 +406,14 @@ def check_empty(x):
     """
     Function to validate if the entry is an empty list or not.
     """
-    if type(x) == list:
+    if type(x) is list:
         return len("".join(x))
-    elif type(x) == float or type(x) == int:
+    elif type(x) is float or type(x) is int:
         return 1
     else:
         return len(x)
 
 
-def edit_html(local_path: str):
-    """
-    Function to include Elucidata logo into the report.
-    """
-    el_image = f"""
-        <a>
-        <img  style="margin-left: 25px;" src={ELUCIDATA_LOGO_URL}
-        width=150" height="70" align="middle">
-    </a>
-    """
-    id_soup = BeautifulSoup(el_image, "html.parser")
-    with open(local_path) as fp:
-        soup = BeautifulSoup(fp, "html.parser")
-    soup.body.insert(1, id_soup)
-    with open(local_path, "wb") as f_output:
-        f_output.write(soup.prettify("utf-8"))
-
-
-def verify_workspace_path(cloud_path: str, credentials: dict) -> tuple:
-    """
-    Function to verify if the workspace path is valid.
-    """
-    access_key_id = credentials["AccessKeyId"]
-    secret_access_key = credentials["SecretAccessKey"]
-    session_token = credentials["SessionToken"]
-    client = S3Client(
-        aws_access_key_id=access_key_id,
-        aws_secret_access_key=secret_access_key,
-        aws_session_token=session_token,
-    )
-    source_path = client.CloudPath(cloud_path)
-    if source_path.exists():
-        return source_path, True
-    else:
-        return source_path, False
-
-
-def check_is_file(self, sts_url: str, workspace_id: int, workspace_path: str) -> bool:
-    """
-    Function to check if the workspace_path is a valid file path existing in the workspace.
-    """
-    creds = self.session.get(sts_url)
-    error_handler(creds)
-    credentials = get_sts_creds(creds.json())
-    if self.session.env == "polly":
-        env_string = "prod"
-    elif self.session.env == "testpolly":
-        env_string = "test"
-    else:
-        env_string = "devenv"
-    bucket = f"mithoo-{env_string}-project-data-v1"
-    s3_path = f"{bucket}/{workspace_id}/"
-    s3_path = f"s3://{make_path(s3_path, workspace_path)}"
-    tuple_output = verify_workspace_path(s3_path, credentials)
-    source_path = tuple_output[0]
-    status = tuple_output[1]
-    if status is True:
-        isFile = source_path.is_file()
-        return isFile
-    return status
-
-
-def split_workspace_path(absolute_path: str) -> tuple:
-    """
-    Function to separate workspace_id and workspace_path from s3 path
-    """
-    contents = absolute_path.split("/")
-    workspace_id = contents[0]
-    workspace_path = contents[1]
-    for item in range(2, len(contents)):
-        workspace_path = make_path(workspace_path, contents[item])
-    return workspace_id, workspace_path
-
-
-def make_private_link(
-    workspace_id: int, workspace_path: str, constant_url: str, report_id: str
-) -> str:
-    """
-    Function to construct and return a private link for a file in workspace.
-    """
-    # encoding the workspace_path for any special character that might pe present in the file_name,
-    # example: 18891/report@name.html
-    parsed_path = urllib.parse.quote(workspace_path)
-    file_element = {"path": f"/projects/{workspace_id}/files/{parsed_path}"}
-    if report_id:
-        return f"{constant_url}/restricted/file?id={workspace_id}&{urllib.parse.urlencode(file_element)}&report_id={report_id}"
-    else:
-        return f"{constant_url}/restricted/file?id={workspace_id}&{urllib.parse.urlencode(file_element)}"
-
-
-def change_file_access(
-    self,
-    access_key: str,
-    workspace_id: int,
-    workspace_path: str,
-    access_url: str,
-    report_id: str,
-) -> str:
-    """
-    Function to change the file access as per the access_key and returns the final access url
-    """
-    final_url = ""
-    # encoding workspace path in case of special characters
-    parsed_workspace_path = urllib.parse.quote(workspace_path)
-    if access_key == "private":
-        params = {"action": "share", "access_type": "private"}
-        url = f"{self.base_url}/projects/{workspace_id}/files/{parsed_workspace_path}"
-        # API call to change the file access to private
-        response = self.session.get(url, params=params)
-        error_handler(response)
-        final_url = make_private_link(
-            workspace_id, workspace_path, access_url, report_id
-        )
-    else:
-        params = {"action": "share", "access_type": "global"}
-        url = f"{self.base_url}/projects/{workspace_id}/files/{parsed_workspace_path}"
-        # API call to change the file access to public
-        response = self.session.get(url, params=params)
-        error_handler(response)
-        shared_id = response.json().get("data").get("shared_id")
-        final_url = f"{access_url}/shared/file/?id={shared_id}"
-    return final_url
-
-
 def get_user_details(session, base_url):
     """
     Function to get user details
@@ -531,55 +425,6 @@ def get_user_details(session, base_url):
     return user_details
 
 
-def get_user_details_using_aws_cognito(self):
-    """
-    Summary :
-    gets the user details such as sub, aud post verification
-    using aws cognito
-
-    Returns:
-        dictionary of token keys and values
-    """
-    # getting the userpool details
-    pool_url = f"https://api.{self.session.env}.elucidata.io/userpool"
-    pool_details = self.session.get(pool_url)
-    pool_details_json = pool_details.json()
-    user_pool_region = pool_details_json.get("cognito_user_pool_region", "")
-    user_pool_id = pool_details_json.get("cognito_user_pool", "")
-    # cognito_client_id = pool_details_json.get("cognito_client_id", "")
-    session_cookies_dict = self.session.cookies.get_dict()
-    id_token = session_cookies_dict.get("idToken", "")
-    session_headers_dict = self.session.headers
-    cookie = session_headers_dict.get("Cookie", "")
-    # cookie is a string
-    if "refreshToken=" in cookie:
-        refresh_token = cookie.split("refreshToken=")[1]
-    else:
-        print("refresh token not presen in header cookie")
-        refresh_token = ""
-
-    # Getting Headers
-    # headers = jwt.get_unverified_header(id_token)
-    kid = get_value_from_idToken(id_token, "kid")
-
-    # Getting Pool Url
-    verify_url = generate_aws_pool_url(user_pool_region, user_pool_id)
-
-    kargs = {"issuer": verify_url}
-    kargs["options"] = {"verify_at_hash": False}
-    kargs["audience"] = get_value_from_idToken(id_token, "aud")
-    keys = aws_key_dict(user_pool_region, user_pool_id)
-
-    # Get Public Key
-    public_key = get_public_key(keys, kid)
-
-    decoded_token = jwt.decode(id_token, public_key, **kargs)
-    decoded_token["id_token"] = id_token
-    decoded_token["refresh_token"] = refresh_token
-
-    return decoded_token
-
-
 def generate_aws_pool_url(user_pool_region, user_pool_id):
     verify_url = "https://cognito-idp.{}.amazonaws.com/{}".format(
         user_pool_region, user_pool_id
@@ -652,67 +497,6 @@ def aws_key_dict(region, user_pool_id):
     return result
 
 
-def get_value_from_idToken(id_token: str, key: str) -> dict:
-    decoded_id_token_dict = decode_id_token(id_token)
-    return decoded_id_token_dict.get(key, "")
-
-
-def decode_id_token(tokenID: str) -> dict:
-    """
-    Summary: decodes the token id
-    this method is to decode the tokenID which consists of 3 parts.
-    the first 2 parts can be base64 decoded
-    first part contains the kid and alg details.
-    second part contains cognito:user details (AWS) such as sub, username
-    aud, name, exp, email etc.
-
-    Args:
-        tokenId : str
-
-    Returns:
-        dict of keys and values from decoded idToken
-    """
-    decoded_idToken_dict = {}
-    try:
-        for element in tokenID.split(".")[:2]:
-            # the element if lacking "padding" can give 'incorrect padding' error.
-            # to handle that we have add the extra padding of '='
-            # the len of the element should always be a multiple of 4.
-            # https://stackoverflow.com/questions/2941995/python-ignore-incorrect-padding-error-when-base64-decoding
-
-            decoded_str = base64.b64decode(element + "=" * (-len(element) % 4))
-            decoded_idToken_dict.update(json.loads(decoded_str.decode("utf-8")))
-    except Exception as err:
-        print("something wrong with idToken")
-        raise err
-    return decoded_idToken_dict
-
-
-def makeRequestCookieForPollyJob(job_data: json) -> json:
-    """
-    Generates the cookie required request for polly job API
-
-    Args:
-        Job data : Dict
-    Returns:
-        request_cookie: generated cookie string
-    """
-
-    secret_env_vars = job_data.get("secret_env", "")
-
-    aud = secret_env_vars.get("POLLY_AUD", "")
-    sub = secret_env_vars.get("POLLY_SUB", "")
-    idToken = secret_env_vars.get("POLLY_ID_TOKEN", "")
-    refreshToken = secret_env_vars.get("POLLY_REFRESH_TOKEN", "")
-    idTokenCookie = f"CognitoIdentityServiceProvider.{aud}.{sub}.idToken={idToken}"
-    refreshCookie = (
-        f"CognitoIdentityServiceProvider.{aud}.{sub}.refreshToken={refreshToken}"
-    )
-    request_coookie = f"{idTokenCookie};{refreshCookie}"
-    # print("returning request_coookie: " + request_coookie)
-    return request_coookie
-
-
 def workspaces_permission_check(self, workspace_id) -> bool:
     """
     Function to check access of a user for a given workspace id.
@@ -744,23 +528,6 @@ def workspaces_permission_check(self, workspace_id) -> bool:
     return False
 
 
-def verify_workspace_details(self, workspace_id, workspace_path, sts_url) -> None:
-    """
-    Function to check and verify workspace permissions and workspace path.
-    """
-    access_workspace = workspaces_permission_check(self, workspace_id)
-    if not access_workspace:
-        raise AccessDeniedError(
-            detail=f"Access denied to workspace-id - {workspace_id}"
-        )
-    is_file = check_is_file(self, sts_url, workspace_id, workspace_path)
-    if not is_file:
-        raise paramException(
-            title="Param Error",
-            detail="The given workspace path does not represent a file. Please try again.",
-        )
-
-
 def return_entity_type(data_source: str, cohort_info: json) -> str:
     """
     Function to return entity type based on the cohort info present in public assets
@@ -815,7 +582,7 @@ def parameter_check_for_repo_id(repo_id):
             title="Param Error",
             detail="repo_id should not be empty",
         )
-    elif type(repo_id) != str and type(repo_id) != int:
+    elif type(repo_id) is not str and type(repo_id) is not int:
         raise paramException(
             title="Param Error",
             detail="repo_id should be str or int",
@@ -919,71 +686,6 @@ def upload_html_file(
         raise e
 
 
-def get_report_id(session, workspace_id: int, workspace_path: str) -> str:
-    """
-    Function to return report-id of the given workspace path.
-    """
-    s3_key = make_path(workspace_id, workspace_path)
-    workspace_endpoint_url = (
-        f"https://v2.api.{session.env}.elucidata.io/v3/workspace/_search"
-    )
-    payload = {
-        "from": 0,
-        "size": 20,
-        "query": {
-            "bool": {
-                "should": [
-                    {
-                        "bool": {
-                            "must": [
-                                {"term": {"_index": {"value": "{document_index}"}}},
-                                {
-                                    "term": {
-                                        "workspace_id": {"value": f"{workspace_id}"}
-                                    }
-                                },
-                                {"term": {"s3_key": {"value": f"{s3_key}"}}},
-                            ]
-                        }
-                    },
-                    {
-                        "bool": {
-                            "must": [
-                                {"term": {"_index": {"value": "{workspace_index}"}}},
-                                {
-                                    "nested": {
-                                        "path": "permissions",
-                                        "query": {
-                                            "match": {
-                                                "permissions.user_id": {
-                                                    "query": "{user_id}"
-                                                }
-                                            }
-                                        },
-                                    }
-                                },
-                            ]
-                        }
-                    },
-                ]
-            }
-        },
-    }
-    response = session.post(workspace_endpoint_url, data=json.dumps(payload))
-    error_handler(response)
-    response_data = response.json()
-    hits = response_data.get("hits", {}).get("hits")
-    if not (hits and isinstance(hits, list)):
-        raise paramException(
-            title="Param Error",
-            detail="Incorrect workspace path / workspace_id provided. Please try again.",
-        )
-    # getting the first and only element of the list that contains data
-    data = hits[0]
-    report_id = data.get("_source", {}).get("report_id")
-    return report_id
-
-
 def get_folder_list_from_list_of_filepaths(filenames_fullpath_list: list) -> list:
     """
     gives back only the folders from a list of filepaths provided.
@@ -1031,3 +733,27 @@ def remove_prefix(text: str, prefix: str) -> str:
         slice_obj = slice(len(prefix), len(text))
         return text[slice_obj]
     return text
+
+
+def find_between(s, first, last) -> str:
+    """Function to find the elements between first and last boundary elements
+    Using this function, a substring can be extract which is between two elements
+    first and last
+    Reference
+    https://stackoverflow.com/questions/3368969/find-string-between-two-substrings
+
+    Args:
+        s (str): main string from
+        first (str): first boundary elements
+        last (str): last boundary elements
+
+    Returns:
+        str: substring b/w boundary elements
+    """
+    try:
+        start = s.index(first) + len(first)
+        end = s.index(last, start)
+        return s[start:end]
+    except ValueError:
+        # write proper error message here
+        raise Exception("Error extracting substring. Please contact admin")
