@@ -4,6 +4,8 @@ from polly.errors import (
     error_handler,
     InvalidPathException,
     AccessDeniedError,
+    InvalidWorkspaceDetails,
+    InvalidDirectoryPath,
 )
 from polly import helpers
 from polly import constants as const
@@ -11,7 +13,7 @@ import logging
 import pandas as pd
 import json
 import os
-from polly.help import example, doc
+from polly.help import example
 from polly.tracking import Track
 
 
@@ -23,13 +25,12 @@ initialize a object that can use all function and methods of Workspaces class.
     Args:
         token (str): Authentication token from polly
     Usage:
-        from polly.Workspaces import Workspaces
+        from polly.workspaces import Workspaces
 
         workspaces = Workspaces(token)
     """
 
     example = classmethod(example)
-    doc = classmethod(doc)
 
     def __init__(self, token=None, env="", default_env="polly") -> None:
         # check if COMPUTE_ENV_VARIABLE present or not
@@ -249,9 +250,9 @@ initialize a object that can use all function and methods of Workspaces class.
         Function to upload files/folders to workspaces.
         Args:
               workspace_id (int) : id of the workspace where file need to uploaded
-              workspace_path (str) : path where the file is to be uploaded.
-              Creates the folder if provided folder path doesn't exist.
-              local_path (str) : uploaded file path
+              workspace_path (str) : path where the file/folder is to be uploaded.
+              The workspace path should be prefixed with "polly://". Creates the folder if provided folder path doesn't exist.
+              local_path (str) : uploaded file/folder path
 
         Raises:
               InvalidParameterException: when the parameter like workspace id is invalid
@@ -273,25 +274,23 @@ initialize a object that can use all function and methods of Workspaces class.
             raise AccessDeniedError(
                 detail=f"Access denied to workspace-id - {workspace_id}"
             )
-        sts_url = f"{self.base_url}/projects/{workspace_id}/credentials/files"
-        creds = self.session.get(sts_url)
-        error_handler(creds)
-        credentials = helpers.get_sts_creds(creds.json())
-        bucket = f"mithoo-{self.env_string}-project-data-v1"
-        s3_path = f"{bucket}/{workspace_id}/"
-        s3_path = f"s3://{helpers.make_path(s3_path, workspace_path)}"
+        if workspace_path.startswith("polly://"):
+            workspace_path = workspace_path.split("polly://")[1]
+        s3_path, credentials = self._s3_util(workspace_id, workspace_path)
         helpers.upload_to_S3(s3_path, local_path, credentials)
         logging.basicConfig(level=logging.INFO)
         logging.info(f"Upload successful on workspace-id={workspace_id}.")
 
     @Track.track_decorator
-    def download_from_workspaces(self, workspace_id: int, workspace_path: str) -> None:
+    def download_from_workspaces(
+        self, workspace_id: int, workspace_path: str, local_path: str
+    ) -> None:
         """
         Function to download files/folders from workspaces.
         A message will be displayed on the status of the operation.
         Args:
               workspace_id (int) : Id of the workspace where file needs to uploaded
-              workspace_path (str) : Downloaded file on workspace
+              workspace_path (str) : Downloaded file on workspace. The workspace path should be prefixed with "polly://"
         Returns:
               None
         Raises:
@@ -309,6 +308,76 @@ initialize a object that can use all function and methods of Workspaces class.
             raise AccessDeniedError(
                 detail=f"Access denied to workspace-id - {workspace_id}"
             )
+        isExists = os.path.isdir(local_path)
+        if not isExists:
+            raise InvalidDirectoryPath(local_path)
+        if workspace_path.startswith("polly://"):
+            workspace_path = workspace_path.split("polly://")[1]
+        s3_path, credentials = self._s3_util(workspace_id, workspace_path)
+        helpers.download_from_S3(s3_path, workspace_path, credentials, local_path)
+        logging.basicConfig(level=logging.INFO)
+        logging.info(f"Download successful to path={local_path}")
+
+    def sync_data(self, workspace_id: int, source_path: str, destination_path: str):
+        """
+        Function to sync directory to or from workspaces and local.
+        Args:
+              workspace_id (int) : id of the workspace which is used for sync
+              source_path (str) : path for the directory that is to be used for sync.
+              Can be local path or a workspace path. The workspace path should be prefixed with "polly://".
+              destination_path (str) : path for the directory that is to be used for sync.
+              Can be local path or a workspace path.
+              The workspace path should be prefixed with "polly://".Creates the folder if provided folder path doesn't exist.
+
+        Raises:
+              InvalidParameterException: when the parameter like workspace id is invalid
+              InvalidDirectoryPath: when the folder to path is invalid
+              InvalidWorkspaceDetails: when the workspace path is not prefixed with "polly://"
+        """
+        if not (workspace_id and isinstance(workspace_id, int)):
+            raise InvalidParameterException("workspace_id")
+        if not (source_path and isinstance(source_path, str)):
+            raise InvalidParameterException("local_path")
+        if not (destination_path and isinstance(destination_path, str)):
+            raise InvalidParameterException("workspace_path")
+        if source_path.startswith("polly://"):
+            self._sync_local(workspace_id, source_path, destination_path)
+        elif destination_path.startswith("polly://"):
+            self._sync_s3(workspace_id, source_path, destination_path)
+        else:
+            raise InvalidWorkspaceDetails()
+
+    def _sync_s3(self, workspace_id, source_path, destination_path):
+        """
+        Function to sync s3 directory with the local directory
+        """
+        isExists = os.path.isdir(source_path)
+        if not isExists:
+            raise InvalidDirectoryPath(source_path)
+        workspace_path = destination_path.split("polly://")[1]
+        s3_path, credentials = self._s3_util(workspace_id, workspace_path)
+        helpers.upload_to_S3(s3_path, source_path, credentials)
+        logging.basicConfig(level=logging.INFO)
+        logging.info(f"Sync successful on workspace-id={workspace_id}.")
+
+    def _sync_local(self, workspace_id, workspace_path, local_path):
+        """
+        Function to sync local directory with the s3 path
+        """
+        isExists = os.path.isdir(local_path)
+        if not isExists:
+            raise InvalidDirectoryPath(local_path)
+
+        workspace_path = workspace_path.split("polly://")[1]
+        s3_path, credentials = self._s3_util(workspace_id, workspace_path)
+        helpers.download_from_S3(s3_path, workspace_path, credentials, local_path)
+        logging.basicConfig(level=logging.INFO)
+        logging.info(f"Sync successful on workspace-id={workspace_id}.")
+
+    def _s3_util(self, workspace_id, workspace_path):
+        """
+        Function to return s3 client credentials and s3_path
+        """
         sts_url = f"{self.base_url}/projects/{workspace_id}/credentials/files"
         creds = self.session.get(sts_url)
         error_handler(creds)
@@ -316,4 +385,4 @@ initialize a object that can use all function and methods of Workspaces class.
         bucket = f"mithoo-{self.env_string}-project-data-v1"
         s3_path = f"{bucket}/{workspace_id}/"
         s3_path = f"s3://{helpers.make_path(s3_path, workspace_path)}"
-        helpers.download_from_S3(s3_path, workspace_path, credentials)
+        return s3_path, credentials
