@@ -5,11 +5,12 @@ from polly.errors import (
     InvalidParameterException,
     UnauthorizedException,
     InvalidPathException,
+    InvalidWorkspaceDetails,
 )
 import polly
 import test_constants as test_const
 
-key = "POLLY_REFRESH_TOKEN"
+key = "POLLY_API_KEY"
 token = os.getenv(key)
 response_json = test_const.WORKSPACE_RESPONSE_JSON
 
@@ -161,6 +162,7 @@ def test_download_from_workspaces(mocker, mock_workspaces_permission_check_fixtu
     valid_workspace_path = "path"
     invalid_workspace_path = ["workspace_path"]
     valid_workspace_path = "workspace_path"
+    local_path = "./"
     mocker = mock_workspaces_permission_check_fixture
     mocker.patch(
         polly.helpers.__name__ + ".get_sts_creds",
@@ -173,15 +175,67 @@ def test_download_from_workspaces(mocker, mock_workspaces_permission_check_fixtu
     response = mocker.patch.object(obj.session, "get")
     response.return_value.status_code = 200
     response.return_value.json.return_value = response_json
-    result = obj.download_from_workspaces(valid_workspace_id, valid_workspace_path)
+    result = obj.download_from_workspaces(
+        valid_workspace_id, valid_workspace_path, local_path
+    )
     assert result is None
     with pytest.raises(
         InvalidParameterException,
         match=r".* Invalid Parameters .*",
     ):
-        obj.download_from_workspaces(invalid_workspace_id, valid_workspace_path)
+        obj.download_from_workspaces(
+            invalid_workspace_id, valid_workspace_path, local_path
+        )
     with pytest.raises(
         InvalidParameterException,
         match=r".* Invalid Parameters .*",
     ):
-        obj.download_from_workspaces(valid_workspace_id, invalid_workspace_path)
+        obj.download_from_workspaces(
+            valid_workspace_id, invalid_workspace_path, local_path
+        )
+
+
+def test_sync_data(mocker, mock_workspaces_permission_check_fixture):
+    obj = workspaces.Workspaces(token)
+    invalid_workspace_id = "12"
+    valid_workspace_id = 12
+    valid_workspace_path = "polly://path"
+    invalid_workspace_path = ["workspace_path"]
+    local_path = "folder/"
+    mocker = mock_workspaces_permission_check_fixture
+    mocker.patch(
+        polly.helpers.__name__ + ".get_sts_creds",
+        return_value=True,
+    )
+    mocker.patch(
+        polly.helpers.__name__ + ".upload_to_S3",
+        return_value=True,
+    )
+    mocker.patch(
+        polly.helpers.__name__ + ".download_from_S3",
+        return_value=True,
+    )
+    mocker.patch(
+        "os.path.isdir",
+        return_value=True,
+    )
+    response = mocker.patch.object(obj.session, "get")
+    response.return_value.status_code = 200
+    response.return_value.json.return_value = response_json
+    result = obj.sync_data(valid_workspace_id, valid_workspace_path, local_path)
+    assert result is None
+    with pytest.raises(
+        InvalidParameterException,
+        match=r".* Invalid Parameters .*",
+    ):
+        obj.sync_data(invalid_workspace_id, valid_workspace_path, local_path)
+    with pytest.raises(
+        InvalidParameterException,
+        match=r".* Invalid Parameters .*",
+    ):
+        obj.sync_data(valid_workspace_id, invalid_workspace_path, local_path)
+    with pytest.raises(
+        InvalidWorkspaceDetails,
+        match=r".* path should start with 'polly://'. .*",
+    ):
+        obj.sync_data(valid_workspace_id, local_path, local_path)
